//
//  PlayerFlutterView.swift
//  KalturaNetKit
//

import Flutter

final class PlayerFlutterViewFactory: NSObject, FlutterPlatformViewFactory {

  private let messenger: FlutterBinaryMessenger

  init(binaryMessenger: FlutterBinaryMessenger) {
    self.messenger = binaryMessenger
    super.init()
  }

  func create(withFrame frame: CGRect, viewIdentifier viewId: Int64, arguments args: Any?) -> FlutterPlatformView {

    let flutterChannel = FlutterMethodChannel(
      name: String(format: "video360Kaltura_%@", String(viewId)),
      binaryMessenger: messenger
    )

    let playerFlutterView = PlayerFlutterView(
      frame: frame,
      viewId: viewId,
      args: args as? [String : Any],
      channel: flutterChannel.setMethodCallHandler
    )

    return playerFlutterView
  }

  func createArgsCodec() -> FlutterMessageCodec & NSObjectProtocol {
    return FlutterStandardMessageCodec.sharedInstance()
  }

}

final class PlayerFlutterView: NSObject, FlutterPlatformView {

  private let frame : CGRect
  private let viewId : Int64
  private let args: Dictionary<String, Any>?

  init(frame: CGRect, viewId: Int64, args: [String: Any]?, channel: (FlutterMethodCallHandler?) -> Void) {
    self.frame = frame
    self.viewId = viewId
    self.args = args
    super.init()
    channel(callHandler)
  }

  func view() -> UIView {

    let rect = CGRect(
      x: args?["x"] as? CGFloat ?? frame.origin.x,
      y: args?["y"] as? CGFloat ?? frame.origin.y,
      width: args?["width"] as? CGFloat ?? frame.size.width,
      height: args?["height"] as? CGFloat ?? frame.size.height
    )

    let view: UIView = NativeView(
      frame:rect
    )

    return view
  }

  private func callHandler(call: FlutterMethodCall, result: FlutterResult) {
    print("[DEBUG] \(call.method)")
    print("[DEBUG] \(String(describing: call.arguments))")
  }

}

final class NativeView: UIView {

  override init(frame: CGRect) {
    super.init(frame: frame)
    debugLayout()
  }

  required init?(coder: NSCoder) {
    super.init(coder: coder)
    debugLayout()
  }

  private func debugLayout() {
    backgroundColor = UIColor.clear

    layer.borderWidth = 1
    layer.borderColor = UIColor.red.cgColor
  }

}
